package cn.fw.valhalla.rpc.shirasawa;

import cn.fw.data.base.domain.common.Message;
import cn.fw.shirasawa.sdk.api.FollowApiService;
import cn.fw.shirasawa.sdk.enums.BusinessTypeEnum;
import cn.fw.shirasawa.sdk.param.FollowChangeUserDTO;
import cn.fw.shirasawa.sdk.param.FollowGenerateDTO;
import cn.fw.shirasawa.sdk.param.TaskCompleteDTO;
import cn.fw.shirasawa.sdk.param.TerminationDTO;
import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.rpc.shirasawa.dto.ClueStopDTO;
import cn.fw.valhalla.rpc.shirasawa.dto.FollowInitDTO;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.Objects;
import java.util.Optional;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * 跟进系统rpc
 *
 * @author : kurisu
 * @version : 1.0
 * @className : ShirasawaRpcService
 * @description : 跟进系统rpc
 * @date : 2022-07-22 14:50
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class ShirasawaRpcService {

    private final FollowApiService followApiService;

    /**
     * 完成/终止跟进任务
     *
     * @param dto
     */
    public boolean stopTask(final ClueStopDTO dto) {
        try {
            if (Objects.isNull(dto.getType())) {
                return true;
            }
            if (!dto.isTermination()) {
                TaskCompleteDTO completeDTO = new TaskCompleteDTO();
                BeanUtils.copyProperties(dto, completeDTO);
                completeDTO.setType(dto.getType());
                completeDTO.setBusinessType(dto.getBusinessType());
                Message<Void> completeMsg = followApiService.completeTask(completeDTO);
                log.info("followApiService.completeTask: msg.code={}, msg.result={}", completeMsg.getCode(), completeMsg.getResult());
                return 0 == completeMsg.getCode();
            }
            TerminationDTO terminationDTO = new TerminationDTO();
            BeanUtils.copyProperties(dto, terminationDTO);
            terminationDTO.setType(dto.getType());
            terminationDTO.setBusinessType(dto.getBusinessType());
            terminationDTO.setReason(dto.getReason().getValue());
            Message<Void> terminationMsg = followApiService.terminationTask(terminationDTO);
            log.info("followApiService.terminationTask: msg.code={}, msg.result={}", terminationMsg.getCode(), terminationMsg.getResult());
            return 0 == terminationMsg.getCode();
        } catch (Exception e) {
            log.error("停止跟进任务失败。detailId: [{}]", dto.getDetailId(), e);
        }
        return false;
    }

    /**
     * 创建跟进系统原始数据
     *
     * @param dto
     */
    public void createFollowData(final FollowInitDTO dto) {
        FollowGenerateDTO followGenerateDTO = new FollowGenerateDTO();
        BeanUtils.copyProperties(dto, followGenerateDTO);
        followGenerateDTO.setType(dto.getType());
        followGenerateDTO.setBusinessType(dto.getBusinessType());
        followGenerateDTO.setNoteMap(dto.getNoteMap());
        final Message<Void> msg = followApiService.generate(followGenerateDTO);
        BV.isTrue(msg.isSuccess(), msg::getResult);
    }

    /**
     * 变更跟进人数据
     *
     * @param dto
     * @param nextTime
     */
    public void changeFollowData(final FollowInitDTO dto, LocalDate nextTime) {
        FollowChangeUserDTO followChangeUserDTO = new FollowChangeUserDTO();
        followChangeUserDTO.setType(dto.getType());
        followChangeUserDTO.setBusinessType(dto.getBusinessType());
        followChangeUserDTO.setDetailId(dto.getDetailId());
        followChangeUserDTO.setShopId(dto.getShopId());
        followChangeUserDTO.setUserId(dto.getUserId());
        followChangeUserDTO.setRecommendNextTime(DateUtil.localDate2Date(nextTime));
        final Message<Void> msg = followApiService.taskChangeUser(followChangeUserDTO);
        BV.isTrue(msg.isSuccess(), msg::getResult);
    }

    /**
     * 变更下次跟进时间
     *
     * @param dataType dataType
     * @param detailId detailId
     * @param nextTime nextTime
     */
    public void updateNextTime(Integer dataType,
                               String detailId,
                               LocalDate nextTime) {
        long time = DateUtil.localDate2Date(nextTime).getTime();
        final Message<Void> msg = followApiService.updateFollowNextTime(dataType, detailId, time, BusinessTypeEnum.AS.getValue());
        BV.isTrue(msg.isSuccess(), msg::getResult);
    }

    /**
     * 查询是否有未完成的待办
     *
     * @param userId
     * @return
     */
    public Boolean hasOngoingFollow(Long userId) {
        try {
            if (Objects.isNull(userId)) {
                return null;
            }
            Message<Long> message = followApiService.queryRecordRemaining(userId, BusinessTypeEnum.AS.getValue());
            if (message.isSuccess()) {
                long data = Optional.ofNullable(message.getData()).orElse(0L);
                return data > 0;
            }
        } catch (Exception e) {
            log.error("查询未完成的待办失败。userId: [{}]", userId, e);
        }
        return null;
    }
}
