package cn.fw.valhalla.controller.task;

import cn.fw.valhalla.component.producer.ClueChangeProducer;
import cn.fw.valhalla.component.producer.RenewalSwitchProducer;
import cn.fw.valhalla.domain.db.follow.ClueTask;
import cn.fw.valhalla.domain.db.follow.FollowClue;
import cn.fw.valhalla.domain.db.pub.PubCluePool;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import cn.fw.valhalla.domain.enums.TaskDefeatTypeEnum;
import cn.fw.valhalla.domain.enums.TaskStateEnum;
import cn.fw.valhalla.sdk.enums.CustomerFollowTypeEnum;
import cn.fw.valhalla.sdk.result.ClueChangeResult;
import cn.fw.valhalla.service.data.ClueTaskService;
import cn.fw.valhalla.service.data.FollowClueService;
import cn.fw.valhalla.service.data.PubCluePoolService;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.data.redis.core.BoundSetOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * 线索变更后的通知事件
 *
 * @author : kurisu
 * @version : 1.0
 * @className : ClueChangeNoticeTask
 * @description : 线索变更后的通知事件
 * @date : 2022-12-07 11:45
 */
@Component
@Slf4j
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
public class ClueChangeNoticeTask {
    private final ClueChangeProducer clueChangeProducer;
    private final RenewalSwitchProducer renewalSwitchProducer;
    private final ClueTaskService clueTaskService;
    private final FollowClueService followClueService;
    private final PubCluePoolService pubCluePoolService;
    private final StringRedisTemplate redisTemplate;
    @Value("${spring.cache.custom.global-prefix}:follow:clue:change")
    @Getter
    private String clueChangeKeyPrefix;

    @Autowired
    public ClueChangeNoticeTask(final ClueChangeProducer clueChangeProducer,
                                final RenewalSwitchProducer renewalSwitchProducer,
                                final ClueTaskService clueTaskService,
                                final FollowClueService followClueService,
                                final PubCluePoolService pubCluePoolService,
                                final StringRedisTemplate redisTemplate) {
        this.clueChangeProducer = clueChangeProducer;
        this.renewalSwitchProducer = renewalSwitchProducer;
        this.clueTaskService = clueTaskService;
        this.followClueService = followClueService;
        this.pubCluePoolService = pubCluePoolService;
        this.redisTemplate = redisTemplate;
    }

    @Scheduled(initialDelay = 1000 * 30, fixedRate = 1000 * 60)
    public void sendStopNotice() {
        List<String> failList = new ArrayList<>();
        final BoundSetOperations<String, String> operations = redisTemplate.boundSetOps(generateStopKey());
        String pop;
        while ((pop = operations.pop()) != null) {
            try {
                final Long taskId = Long.valueOf(pop);
                ClueTask task = clueTaskService.getById(taskId);
                if (Objects.isNull(task)) {
                    continue;
                }
                if (FollowTypeEnum.PL.equals(task.getType())) {
                    sendPubMQ(task);
                    return;
                }
                sendNormalMQ(task);
            } catch (Exception ex) {
                failList.add(pop);
                log.error(ex.getMessage(), ex);
            }
        }
        if (!CollectionUtils.isEmpty(failList)) {
            operations.add(failList.toArray(new String[0]));
        }
    }

    private void sendNormalMQ(ClueTask task) {
        FollowClue clue = followClueService.getById(task.getClueId());
        if (Objects.isNull(clue)) {
            return;
        }
        ClueChangeResult result = new ClueChangeResult();
        result.setFrameNo(clue.getVin());
        result.setGroupId(clue.getGroupId());
        TaskDefeatTypeEnum reason = task.getReason();

        if (FollowTypeEnum.RM.equals(clue.getClueType())) {
            result.setClueType(CustomerFollowTypeEnum.RM.getValue());
        } else if (FollowTypeEnum.IR.equals(clue.getClueType())) {
            result.setClueType(CustomerFollowTypeEnum.IR.getValue());
            renewalSwitchProducer.send(clue.getVin(), clue.getEndTime().toLocalDate(), clue.getGroupId());
        } else if (FollowTypeEnum.FM.equals(clue.getClueType())) {
            result.setClueType(CustomerFollowTypeEnum.FM.getValue());
        }

        if (TaskStateEnum.COMPLETE.equals(task.getState())) {
            result.setChangeType(ClueChangeResult.ChangeType.COMPLETE);
        } else {
            processChangeType(reason, result);
        }

        clueChangeProducer.send(result);
    }

    private void sendPubMQ(ClueTask task) {
        PubCluePool clue = pubCluePoolService.getById(task.getClueId());
        ClueChangeResult result = new ClueChangeResult();
        result.setFrameNo(clue.getVin());
        result.setGroupId(clue.getGroupId());
        TaskDefeatTypeEnum reason = task.getReason();
        result.setClueType(CustomerFollowTypeEnum.PL.getValue());
        if (TaskStateEnum.COMPLETE.equals(task.getState())) {
            result.setChangeType(ClueChangeResult.ChangeType.COMPLETE);
        } else {
            processChangeType(reason, result);
        }
        clueChangeProducer.send(result);
    }

    private void processChangeType(TaskDefeatTypeEnum reason, ClueChangeResult result) {
        if (TaskDefeatTypeEnum.A.equals(reason)) {
            result.setChangeType(ClueChangeResult.ChangeType.ABANDON);
        }
        if (TaskDefeatTypeEnum.F.equals(reason)) {
            result.setChangeType(ClueChangeResult.ChangeType.OTHER_COMPLETE);
        }
        if (TaskDefeatTypeEnum.D.equals(reason)) {
            result.setChangeType(ClueChangeResult.ChangeType.ROLE_CHANGE);
        }
        if (TaskDefeatTypeEnum.C.equals(reason)) {
            result.setChangeType(ClueChangeResult.ChangeType.STOP);
        }
    }

    private String generateStopKey() {
        return String.format("%s:%s", getClueChangeKeyPrefix(), "STOP");
    }
}
