package cn.fw.valhalla.controller.task;

import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.domain.db.customer.AffiliationRecord;
import cn.fw.valhalla.domain.db.pool.PublicCluePool;
import cn.fw.valhalla.domain.db.pool.PublicPool;
import cn.fw.valhalla.domain.dto.CustomerDetailDto;
import cn.fw.valhalla.domain.enums.CustomerChangeTypeEnum;
import cn.fw.valhalla.domain.enums.DefeatReasonEnum;
import cn.fw.valhalla.domain.enums.PublicClueStateEnum;
import cn.fw.valhalla.domain.enums.PublicPoolTypeEnum;
import cn.fw.valhalla.service.bus.cust.CustomerBizService;
import cn.fw.valhalla.service.bus.follow.PCFollowBizService;
import cn.fw.valhalla.service.data.AffiliationRecordService;
import cn.fw.valhalla.service.data.PublicCluePoolService;
import cn.fw.valhalla.service.data.PublicPoolService;
import cn.fw.valhalla.service.data.StammkundePoolService;
import cn.fw.valhalla.service.event.PublicPoolEvent;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author : kurisu
 * @className : PublicClueTask
 * @description : 公共客户线索池处理任务
 * @date: 2021-03-30 17:24
 * @deprecated ⚠️下版本废弃 slice 2.0
 */
@Component
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
@Slf4j
public class PublicClueTask {
    private final PublicCluePoolService publicCluePoolService;
    private final PublicPoolService publicPoolService;
    private final CustomerBizService customerBizService;
    private final StammkundePoolService stammkundePoolService;
    private final PCFollowBizService pcFollowBizService;
    private final ApplicationEventPublisher eventPublisher;
    private final AffiliationRecordService affiliationRecordService;
    private final PlatformTransactionManager platformTransactionManager;
    private final TransactionDefinition transactionDefinition;

    @Autowired
    public PublicClueTask(final PublicCluePoolService publicCluePoolService,
                          final PublicPoolService publicPoolService,
                          final CustomerBizService customerBizService,
                          final StammkundePoolService stammkundePoolService,
                          final PCFollowBizService pcFollowBizService,
                          final ApplicationEventPublisher eventPublisher,
                          final AffiliationRecordService affiliationRecordService,
                          final PlatformTransactionManager platformTransactionManager,
                          final TransactionDefinition transactionDefinition) {
        this.publicCluePoolService = publicCluePoolService;
        this.publicPoolService = publicPoolService;
        this.customerBizService = customerBizService;
        this.stammkundePoolService = stammkundePoolService;
        this.pcFollowBizService = pcFollowBizService;
        this.eventPublisher = eventPublisher;
        this.affiliationRecordService = affiliationRecordService;
        this.platformTransactionManager = platformTransactionManager;
        this.transactionDefinition = transactionDefinition;
    }


    /**
     * 开始任务
     */
    @Scheduled(cron = "0/30 * * * * *")
    public void endClue() {
        try {
            List<PublicCluePool> list = publicCluePoolService.list(Wrappers.<PublicCluePool>lambdaQuery()
                    .eq(PublicCluePool::getState, PublicClueStateEnum.ONGOING)
                    .le(PublicCluePool::getDeadline, DateUtil.startDate(new Date()))
                    .last("limit 0, 1000")
            );
            if (CollectionUtils.isEmpty(list)) {
                return;
            }
            for (PublicCluePool cluePool : list) {
                doSomething(cluePool);
            }
        } catch (Exception e) {
            log.error("定时处理公共客户线索池失败", e);
        }
    }

    @Scheduled(cron = "0/60 * * * * *")
    public void endRecord() {
        try {
            pcFollowBizService.overdueProcessing();
        } catch (Exception e) {
            log.error("定时关闭公共客户线索池跟进记录失败", e);
        }
    }

    public void doSomething(PublicCluePool cluePool) {
        TransactionStatus transactionStatus = platformTransactionManager.getTransaction(transactionDefinition);
        try {
            cluePool.setCloseTime(new Date());
            cluePool.setState(PublicClueStateEnum.DEFEAT);
            Long customerId = cluePool.getCustomerId();
            CustomerDetailDto detailDto = customerBizService.queryById(customerId);
            if (Objects.isNull(detailDto)) {
                return;
            }
            detailDto.setAdviserId(cluePool.getAdviserId());
            detailDto.setAdviserName(cluePool.getAdviserName());
            detailDto.setShopId(cluePool.getShopId());
            AffiliationRecord record = new AffiliationRecord();
            record.setCustomerId(detailDto.getId());
            record.setType(CustomerChangeTypeEnum.DEFEAT);
            record.setDefeatTime(DateUtil.localDateTime2Date(LocalDateTime.now()));
            record.setOriginUserId(cluePool.getAdviserId());
            record.setOriginShopId(cluePool.getShopId());
            record.setCreateTime(DateUtil.localDateTime2Date(LocalDateTime.now()));
            record.setReason("公共客户线索到期战败");
            saveData(cluePool, detailDto, record);
            platformTransactionManager.commit(transactionStatus);
        } catch (Exception e) {
            platformTransactionManager.rollback(transactionStatus);
            throw e;
        }
    }

    public void saveData(PublicCluePool cluePool, CustomerDetailDto detailDto, AffiliationRecord record) {
        PublicPool publicPool = customerBizService.createPublicPool(detailDto, PublicPoolTypeEnum.PD, PublicPoolTypeEnum.PD.getName());
        stammkundePoolService.reject(cluePool.getCustomerId(), cluePool.getGroupId(), DefeatReasonEnum.LC);
        customerBizService.move2PublicPool(detailDto.getId());
        publicPoolService.save(publicPool);
        publicCluePoolService.updateById(cluePool);
        affiliationRecordService.save(record);
        PublicPoolEvent poolEvent = new PublicPoolEvent(detailDto.getFrameNo(), detailDto.getGroupId(), true);
        eventPublisher.publishEvent(poolEvent);
    }
}
