package cn.fw.valhalla.service.bus.cust;

import cn.fw.common.util.ValidationUtils;
import cn.fw.common.web.annotation.DisLock;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.common.utils.StringUtils;
import cn.fw.valhalla.domain.db.customer.AccidentPool;
import cn.fw.valhalla.domain.db.follow.ClueTask;
import cn.fw.valhalla.domain.db.follow.FollowClue;
import cn.fw.valhalla.domain.dto.AccidentPoolDTO;
import cn.fw.valhalla.domain.enums.*;
import cn.fw.valhalla.service.bus.setting.SettingBizService;
import cn.fw.valhalla.service.data.AccidentPoolService;
import cn.fw.valhalla.service.data.ClueTaskService;
import cn.fw.valhalla.service.data.FollowClueService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

import static cn.fw.common.businessvalidator.Validator.BV;
import static cn.fw.valhalla.service.bus.setting.strategy.SettingStrategy.COMMON_BRAND_ID;

/**
 * @author : kurisu
 * @className : AccidentPoolBizService
 * @description : 事故池
 * @date: 2020-08-15 15:20
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class AccidentPoolBizService {
    private final AccidentPoolService accidentPoolService;
    private final ClueTaskService clueTaskService;
    private final FollowClueService followClueService;
    private final SettingBizService settingBizService;


    @Value("${spring.cache.custom.global-prefix}:AccidentPool")
    @Getter
    private String keyPrefix;

    @Value("${follow.todo.ACCode}")
    @Getter
    private String ACCode;


    @DisLock(prefix = "#this.getKeyPrefix()", key = "#currentUser.groupId + ':' + #poolDTO.plateNo", message = "请勿重复提交")
    @Transactional(rollbackFor = Exception.class)
    public Long add2Pool(LoginAuthBean currentUser, AccidentPoolDTO poolDTO) {
        if (Long.valueOf(146L).equals(poolDTO.getShopId())) {
            poolDTO.setShopId(67L);
            poolDTO.setShopName("贺兰捷联售后店");
        }
        boolean isPlateNo = ValidationUtils.checkCarPlate(poolDTO.getPlateNo());
        BV.isTrue(isPlateNo, () -> "车牌号不正确，请检查是否包含I、O或特殊字符");
        if (!Boolean.TRUE.equals(poolDTO.getForce())) {
            AccidentPool repetition = isRepetition(currentUser.getGroupId(), poolDTO.getPlateNo());
            BV.isNull(repetition, poolDTO.getPlateNo() + "有正在进行的事故车待办");
            if (Objects.nonNull(repetition)) {
                return repetition.getCreateTime().getTime();
            }
        }

        if (StringUtils.isEmpty(poolDTO.getMobile())) {
            poolDTO.setMobile(poolDTO.getReportMobile());
        }
        AccidentPool pool = conversion2db(poolDTO, currentUser);
        pool.setShopId(poolDTO.getShopId());
        pool.setShopName(poolDTO.getShopName());
        BV.notNull(pool.getShopId(), () -> "服务站信息不正确，请确认");
        accidentPoolService.save(pool);
        afterAddPool(pool);
        return 0L;
    }


    @Transactional(rollbackFor = Exception.class)
    public void afterAddPool(final AccidentPool pool) {
        final String plateNo = pool.getPlateNo();
        List<FollowClue> clueList = followClueService.list(Wrappers.<FollowClue>lambdaQuery()
                .eq(FollowClue::getPlateNo, plateNo)
                .eq(FollowClue::getClueType, FollowTypeEnum.AC)
                .eq(FollowClue::getClueState, ClueStatusEnum.ONGOING)
        );
        if (!CollectionUtils.isEmpty(clueList)) {
            for (FollowClue followClue : clueList) {
                closeTask(followClue);
            }
        }
        createClue(pool);
    }

    @Transactional(rollbackFor = Exception.class)
    public void closeTask(FollowClue clue) {
        clue.setClueState(ClueStatusEnum.FAILURE);
        ClueTask clueTask = clueTaskService.queryOngoingTaskByClueId(clue.getId(), clue.getClueType());
        if (Objects.nonNull(clueTask)) {
            clueTask.setState(TaskStateEnum.DEFEAT);
            clueTask.setReason(TaskDefeatTypeEnum.C);
            clueTask.setCloseTime(LocalDateTime.now());
            clueTask.setRpcSuccess(Boolean.FALSE);
        }
        clueTaskService.updateById(clueTask);
        followClueService.updateById(clue);
    }


    /**
     * 新增线索
     *
     * @param pool
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public void createClue(final AccidentPool pool) {
        final LocalDateTime now = LocalDateTime.now();
        final FollowClue cluePool = new FollowClue();
        cluePool.setPlateNo(pool.getPlateNo());
        cluePool.setVin(pool.getFrameNo());
        cluePool.setOriginTime(DateUtil.date2LocalDateTime(pool.getCreateTime()));
        cluePool.setClueState(ClueStatusEnum.WAITING);
        cluePool.setClueType(FollowTypeEnum.AC);
        cluePool.setSuggestShopId(pool.getShopId());
        cluePool.setStartTime(now);
        cluePool.setSuggestMobile(pool.getReportMobile());
        settingBizService.querySettingByType(FollowTypeEnum.AC, SettingTypeEnum.FAIL_TIME, pool.getGroupId(), COMMON_BRAND_ID)
                .ifPresent(r -> {
                    long detailValue = Long.valueOf(Optional.ofNullable(r.getDetailValue()).orElse(2));
                    LocalDateTime deadline = calTime(now, Objects.requireNonNull(SettingUnitEnum.ofValue(r.getUnit())), detailValue);
                    cluePool.setEndTime(deadline);
                });
        cluePool.setGroupId(pool.getGroupId());
        cluePool.setBizId(pool.getId());
        cluePool.setCreateTime(new Date());
        followClueService.save(cluePool);
    }

    private AccidentPool conversion2db(AccidentPoolDTO dto, LoginAuthBean currentUser) {
        AccidentPool pool = new AccidentPool();
        BeanUtils.copyProperties(dto, pool);
        pool.setAddress(dto.getReportAddress());
        pool.setCreateTime(new Date());
        pool.setUpdateTime(new Date());
        pool.setGroupId(currentUser.getGroupId());
        return pool;
    }

    private AccidentPool isRepetition(Long groupId, String plateNo) {
        AccidentPool lastOne = accidentPoolService.getOne(Wrappers.<AccidentPool>lambdaQuery()
                .eq(AccidentPool::getPlateNo, plateNo)
                .eq(AccidentPool::getGroupId, groupId)
                .orderByDesc(AccidentPool::getCreateTime)
                .last(" limit 1 ")
        );

        if (Objects.isNull(lastOne)) {
            return null;
        }
        FollowClue lastOngoingClue = followClueService.getOne(Wrappers.<FollowClue>lambdaQuery()
                .eq(FollowClue::getPlateNo, plateNo)
                .eq(FollowClue::getClueType, FollowTypeEnum.AC)
                .eq(FollowClue::getClueState, ClueStatusEnum.ONGOING)
                .eq(FollowClue::getGroupId, groupId)
                .last(" limit 1 ")
        );

        if (Objects.nonNull(lastOngoingClue)) {
            return lastOne;
        }
        return null;
    }


    public LocalDateTime calTime(LocalDateTime baseTime, SettingUnitEnum unit, long value) {
        switch (unit) {
            case HOUR:
                return baseTime.plusHours(value);
            case MONTH:
                return baseTime.plusMonths(value);
            case MINUTE:
                return baseTime.plusMinutes(value);
            default:
                return baseTime.plusDays(value);
        }
    }
}
