package cn.fw.valhalla.service.bus.follow.strategy.impl;

import cn.fw.shirasawa.sdk.enums.BusinessTypeEnum;
import cn.fw.shirasawa.sdk.enums.DataTypeEnum;
import cn.fw.valhalla.common.constant.RoleCode;
import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.common.utils.StringUtils;
import cn.fw.valhalla.domain.db.OriginalData;
import cn.fw.valhalla.domain.db.customer.AccidentPool;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.db.follow.ClueTask;
import cn.fw.valhalla.domain.db.follow.FollowClue;
import cn.fw.valhalla.domain.dto.CustomerDetailDto;
import cn.fw.valhalla.domain.enums.*;
import cn.fw.valhalla.domain.vo.setting.SettingVO;
import cn.fw.valhalla.rpc.ehr.dto.StaffInfoDTO;
import cn.fw.valhalla.rpc.erp.dto.PostUserDTO;
import cn.fw.valhalla.rpc.oop.dto.ShopDTO;
import cn.fw.valhalla.rpc.shirasawa.dto.FollowInitDTO;
import cn.fw.valhalla.service.bus.follow.strategy.AbstractFollowStrategy;
import cn.fw.valhalla.service.data.AccidentPoolService;
import cn.fw.valhalla.service.event.ClueFromPublicEvent;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.time.Duration;
import java.util.*;
import java.util.concurrent.CompletableFuture;

import static cn.fw.common.businessvalidator.Validator.BV;
import static cn.fw.valhalla.service.bus.setting.strategy.SettingStrategy.COMMON_BRAND_ID;

/**
 * @author : kurisu
 * @className : ACFollowStrategy
 * @description : 事故车策略
 * @date: 2020-08-17 10:48
 */
@Slf4j
@Component
public class ACFollowStrategy extends AbstractFollowStrategy {

    private final AccidentPoolService accidentPoolService;
    private final ApplicationEventPublisher eventPublisher;

    @Autowired
    public ACFollowStrategy(final AccidentPoolService accidentPoolService,
                            final ApplicationEventPublisher eventPublisher) {
        this.accidentPoolService = accidentPoolService;
        this.eventPublisher = eventPublisher;
    }

    @Override
    public FollowTypeEnum getFollowType() {
        return FollowTypeEnum.AC;
    }

    @Override
    public boolean origin2task(OriginalData originalData) {
        this.finishClue(originalData);
        return true;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void startClue(FollowClue followClue) {
        if (!ClueStatusEnum.WAITING.equals(followClue.getClueState())) {
            return;
        }
        AccidentPool accidentPool = accidentPoolService.getById(followClue.getBizId());
        if (Objects.isNull(accidentPool)) {
            return;
        }
        final ClueTask clueTask = createNewTask(followClue);
        List<PostUserDTO> userByRole = userService.getUserByRole(clueTask.getFollowShop(), RoleCode.SGCGJ);
        BV.isNotEmpty(userByRole, () -> "该门店没有事故车跟进人员");
        fillTaskUser(clueTask, userByRole);

        if (StringUtils.isValid(followClue.getVin())) {
            final ClueFromPublicEvent poolEvent = new ClueFromPublicEvent();
            poolEvent.setVin(followClue.getVin());
            poolEvent.setShopId(clueTask.getFollowShop());
            poolEvent.setGroupId(clueTask.getGroupId());
            poolEvent.setUserId(clueTask.getFollowUser());
            poolEvent.setType(PubStandType.AC);
            CompletableFuture.runAsync(() -> eventPublisher.publishEvent(poolEvent));
        }

        followClue.setClueState(ClueStatusEnum.ONGOING);
        clueTaskService.save(clueTask);
        final FollowInitDTO followInitDTO = creteFollowInitDTO(followClue, clueTask);
        followInitDTO.setCustomerName(accidentPool.getName());
        shirasawaRpcService.createFollowData(followInitDTO);
        followClueService.updateById(followClue);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void settingChanged(List<SettingVO> setting, Long groupId) {
        // 更改设置不影响事故车跟进
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void closeTask(ClueTask task) {
        Long clueId = task.getClueId();
        FollowClue clue = followClueService.getById(clueId);
        task.setState(TaskStateEnum.DEFEAT);
        task.setCloseTime(task.getDeadline().minusSeconds(1L));
        task.setReason(TaskDefeatTypeEnum.C);
        boolean rpcSucess = rpcStopTask(task);
        task.setRpcSuccess(rpcSucess);
        if (Objects.nonNull(clue)) {
            clue.setClueState(ClueStatusEnum.FAILURE);
            clue.setCloseTime(task.getCloseTime());
            followClueService.updateById(clue);
        }
        clueTaskService.updateById(task);
    }

    @Override
    protected FollowInitDTO creteFollowInitDTO(FollowClue followClue, ClueTask clueTask) {
        final FollowInitDTO followInitDTO = FollowInitDTO.builder()
                .businessType(BusinessTypeEnum.AS)
                .plateNo(followClue.getPlateNo())
                .type(DataTypeEnum.ofValue(clueTask.getType().getValue()))
                .frameNo(followClue.getVin())
                .contacts(followClue.getSuggestMobile())
                .detailId(String.valueOf(followClue.getId()))
                .generateTime(DateUtil.localDateTime2Date(clueTask.getBeginTime()))
                .deadline(DateUtil.localDateTime2Date(clueTask.getDeadline()))
                .groupId(clueTask.getGroupId())
                .shopId(clueTask.getFollowShop())
                .userId(clueTask.getFollowUser())
                .userName(clueTask.getFollowUserName())
                .bizId(followClue.getVin())
                .build();

        if (StringUtils.isValid(followClue.getVin())) {
            CustomerDetailDto customerDetailDto = customerBizService.queryByFrameNo(followClue.getVin(), followClue.getGroupId());
            if (Objects.nonNull(customerDetailDto)) {
                followInitDTO.setCustomerId(customerDetailDto.getId());
                followInitDTO.setCustomerName(customerDetailDto.getName());
                followInitDTO.setMemberId(customerDetailDto.getMemberId());
            }
        }

        ShopDTO shop = oopService.shop(clueTask.getFollowShop());
        if (Objects.nonNull(shop)) {
            followInitDTO.setShopName(shop.getShopName());
        }
        settingBizService.querySettingByType(getFollowType(), SettingTypeEnum.EFFECTIVE_TIME, clueTask.getGroupId(), COMMON_BRAND_ID)
                .ifPresent(r -> {
                    Integer unit = r.getUnit();
                    int detailValue = Optional.ofNullable(r.getDetailValue()).orElse(0);
                    Duration duration = transferDuration(detailValue, unit);
                    BV.notNull(duration, () -> "设置有误，周期计算不成功");
                    followInitDTO.setFollowDuration(duration);
                });
        Map<String, String> noteMap = createNoteMap(followClue);
        followInitDTO.setNoteMap(noteMap);
        return followInitDTO;
    }

    @Override
    protected Map<String, String> createNoteMap(FollowClue followClue) {
        Map<String, String> dynamicMap = new HashMap<>();
        if (Objects.nonNull(followClue)) {
            String name = "-";
            String insComName = "-";
            Long bizId = followClue.getBizId();
            AccidentPool accidentPool = accidentPoolService.getById(bizId);
            if (Objects.nonNull(accidentPool)) {
                name = StringUtils.isValid(accidentPool.getName()) ? accidentPool.getName() : "-";
                insComName = StringUtils.isValid(accidentPool.getInsurerName()) ? accidentPool.getInsurerName() : "-";
            }
            dynamicMap.put("name", name);
            dynamicMap.put("state", "1");
            dynamicMap.put("plateNo", StringUtils.isValid(followClue.getPlateNo()) ? followClue.getPlateNo() : "-");
            dynamicMap.put("insComName", insComName);
        }
        return dynamicMap;
    }

    /**
     * 线索成交
     *
     * @param originalData
     */
    private void finishClue(OriginalData originalData) {
        String plateNo = originalData.getPlateNo();
        if (StringUtils.isEmpty(plateNo)) {
            Customer customer = null;
            if (StringUtils.isValid(originalData.getFrameNo())) {
                customer = customerService.queryByFrameNo(originalData.getFrameNo(), originalData.getGroupId());
            }
            if (Objects.isNull(customer)) {
                customer = customerService.queryById(originalData.getCustomerId());
            }
            if (Objects.nonNull(customer)) {
                plateNo = customer.getPlateNo();
            }

        }
        if (StringUtils.isEmpty(plateNo)) {
            return;
        }
        FollowClue followClue = followClueService.getOne(Wrappers.<FollowClue>lambdaQuery()
                        .eq(FollowClue::getPlateNo, plateNo)
                        .eq(FollowClue::getClueType, getFollowType())
                        .eq(FollowClue::getClueState, ClueStatusEnum.ONGOING)
                , Boolean.FALSE);
        if (Objects.isNull(followClue)) {
            return;
        }
        ClueTask clueTask = clueTaskService.queryOngoingTaskByClueId(followClue.getId(), followClue.getClueType());
        if (Objects.isNull(clueTask)) {
            return;
        }

        followClue.setClueState(ClueStatusEnum.COMPLETE);
        clueTask.setState(TaskStateEnum.COMPLETE);
        clueTask.setCloseTime(DateUtil.date2LocalDateTime(originalData.getGenerateTime()));
        clueTask.setFinishUser(originalData.getUserId());
        StaffInfoDTO infoDTO = ehrRpcService.queryStaffInfo(originalData.getUserId());
        if (Objects.nonNull(infoDTO)) {
            clueTask.setFinishUserName(infoDTO.getName());
        }
        clueTask.setFinishShop(originalData.getShopId());
        followClueService.updateById(followClue);
        boolean rpcSucess = rpcStopTask(clueTask);
        clueTask.setRpcSuccess(rpcSucess);
        clueTaskService.updateById(clueTask);
    }
}
